/******************************************************************************
 * Author: Phil DeMonaco
 *
 * A simple method which provides error display and graceful shutdown used by
 * both classes.
/******************************************************************************/

#include "errorHandler.h" // header for this class
#include <errno.h>      // check if a system error needs to be printed
#include "stddef.h"     // standard character definitions
#include <stdio.h>      // fprintf
#include <stdlib.h>     // EXIT_FAILURE, etc...

/***********************************************************************
 * Function: printError
 * Purpose: print the given error message and, if necessary terminate
 *
 * Parameter(s):
 *  (char*) message:    base message to be printed
 *  (void*) param:      pointer to message argument (if unused give \0)
 *  (int)   paramType:  the type of parameter to use, CHAR, INT, STRING
 *  (int)   fatal:      should the program terminate?
/***********************************************************************/
void printError(char *message, void *param, int paramType, int fatal)
{
    /* convert param to its actual type based on paramType */
    switch (paramType)
    {
        case CHAR:
            fprintf(stderr, message, ((char*)param)[0]);
            break;
        case INT:
            fprintf(stderr, message, *((int*)param));
            break;
        default:
            fprintf(stderr, message, (char*)param);
    }

    if (errno)
    {
        perror(NULLCHR);      /* print last system error if it exits */
        errno = FALSE;          // clear the error code
    }
    else putchar(NEW_LINE);            /* otherwise newline (perror does this)*/
    if (fatal) exit(EXIT_FAILURE);     /* terminate if error was fatal */
}
